import numpy as np
from datetime import datetime,timedelta
import os
from . import date, solar_wind, stations
from . import misc, Request, Server

def ConeInputs(Stations,customlocations,startaltitude,minaltitude,zenith,azimuth,maxdistance,maxtime,
           serverdata,livedata,vx,vy,vz,by,bz,density,pdyn,Dst,
           G1,G2,G3,W1,W2,W3,W4,W5,W6,kp,Anum,anti,year,
           month,day,hour,minute,second,internalmag,externalmag,
           intmodel,startrigidity,endrigidity,rigiditystep,
           coordsystem,gyropercent,magnetopause,corenum,g,h,MHDfile,MHDcoordsys):
    
    EventDate = datetime(year,month,day,hour,minute,second)
    DateCreate = date.Date(EventDate)
    DateArray = DateCreate.GetDate()

    if anti == "YES":
         AntiCheck = 1
    elif anti == "NO":
         AntiCheck = 0
    else:
         print("Please enter a valid anti value: ""YES"" or ""NO"" ")
         exit()

    if magnetopause == "Sphere":
         Magnetopause = 0
    elif magnetopause == "aFormisano":
         Magnetopause = 1
    elif magnetopause == "Sibeck":
         Magnetopause = 2
    elif magnetopause == "Kobel":
         Magnetopause = 3
    elif magnetopause == "NONE":
         Magnetopause = 99
    else:
         print("Please enter a valid magnetopause model: ""Sphere"", ""aFormisano"", ""Sibeck"", ""Kobel"", ""NONE"" ")
         exit()


    if intmodel == "4RK":
         IntModel = 1
    elif intmodel == "Boris":
         IntModel = 2
    elif intmodel == "Vay":
         IntModel = 3
    elif intmodel == "HC":
         IntModel = 4
    else:
         print("Please enter a valid intmodel model: ""4RK"", ""Boris"", ""Vay"", ""HC"" ")
         exit()

    if serverdata == "ON":
         ServerData = 1
    elif serverdata == "OFF":
         ServerData = 0
    else:
         print("Please enter a valid serverdata value: ""ON"" or ""OFF"" ")
         exit()

    if livedata == "ON":
         LiveData = 1
    elif livedata == "OFF":
         LiveData = 0
    else:
         print("Please enter a valid livedata value: ""ON"" or ""OFF"" ")
         exit()
    
    if internalmag == "NONE":
         Internal = 0
         if not g or not h: 
            g = [0] * 105
            h = [0] * 105
    elif internalmag == "IGRF":
         Internal = 1
         if not g or not h: 
            g = [0] * 105
            h = [0] * 105
    elif internalmag == "Dipole":
         Internal = 2
         if not g or not h: 
            g = [0] * 105
            h = [0] * 105
    elif internalmag == "Custom Gauss":
         Internal = 4
         if not g or not h:
              print("Please enter values for the g and h Gaussian coefficients to use the Custom Gauss option")
              exit()
         elif len(g) != 105:
              print(f"There should be 105 g coefficents in the inputted list, you have entered {len(g)}")
              exit()
         elif len(h) != 105:
              print(f"There should be 105 h coefficents in the inputted list, you have enetered {len(h)}")
    else:
         print("Please enter a valid internalmag model: ""NONE"",""IGRF"",""Dipole"", or ""Custom Gauss""")
         exit()
      
    if externalmag == "NONE":
         External = 0
    elif externalmag == "TSY87short":
         External = 1
    elif externalmag == "TSY87long":
         External = 2
    elif externalmag == "TSY89":
         External = 3
    elif externalmag == "TSY96":
         External = 4
    elif externalmag == "TSY01":
         External = 5
    elif externalmag == "TSY01S":
         External = 6
    elif externalmag == "TSY04":
         External = 7
    elif externalmag == "TSY89_BOBERG":
         External = 8
    elif externalmag == "MHD":
         External = 99
         if not os.path.exists(MHDfile):
            print(f"The file '{MHDfile}' does not exist.")
            exit()
    else:
         print("Please enter a valid externalmag model: ""NONE"", ""TSY87short"",""TSy87long"",""TSY89"",""TSY89_BOBERG"",""TSY96"",""TSY01"",""TSY01S"",""TSY04""")
         exit()

    if coordsystem not in ["GDZ","GEO","GSM","GSE","SM","GEI","MAG","SPH","RLL"]:
         print("Please select a valid coordsystem: ""GDZ"", ""GEO"", ""GSM"", ""GSE"", ""SM"", ""GEI"", ""MAG"", ""SPH"", ""RLL""")
         exit()


    misc.DataCheck(ServerData,LiveData,EventDate)

    IOPTinput = misc.IOPTprocess(kp)
    KpS = 0

    if ServerData == 1:
         if int(EventDate.year) >= 1981:
              Server.DownloadServerFile(int(EventDate.year))
         elif int(EventDate.year) < 1981 and int(EventDate.year) > 1963:
              Server.DownloadServerFileLowRes(int(EventDate.year))
         else:
              print("Server data only valid for 1963 to present, please enter a valid date.")
         ByS, BzS, VS, DensityS, PdynS, KpS, DstS, G1S, G2S, G3S, W1S, W2S, W3S, W4S, W5S, W6S = Server.GetServerData(EventDate,External)
         IOPTinput = misc.IOPTprocess(KpS)
         WindCreate = solar_wind.Solar_Wind(VS, vy, vz, ByS, BzS, DensityS, PdynS, DstS, G1S, G2S, G3S, W1S, W2S, W3S, W4S, W5S, W6S, KpS)
         WindArray = WindCreate.GetWind()
         
    if LiveData == 1:
         misc.DateCheck(EventDate)
         DstLive, VxLive, DensityLive, ByLive, BzLive, IOPTLive, G1Live, G2Live, G3Live, KpLive = Request.Get_Data(EventDate)
         PdynLive = misc.Pdyn_comp(DensityLive,VxLive)
         IOPTinput = IOPTLive
         WindCreate = solar_wind.Solar_Wind(VxLive, vy, vz, ByLive, BzLive, DensityLive, PdynLive, DstLive, G1Live, G2Live, G3Live, W1, W2, W3, W4, W5, W6, KpLive)
         WindArray = WindCreate.GetWind()

    if ServerData == 0 and LiveData == 0:
          if vx > 0:
               vx = -1*vx
          WindCreate = solar_wind.Solar_Wind(vx, vy, vz, by, bz, density, pdyn, Dst, G1, G2, G3, W1, W2, W3, W4, W5, W6, kp)
          WindArray = WindCreate.GetWind()


    RigidityArray = [startrigidity,endrigidity,rigiditystep]

    MagFieldModel = np.array([Internal,External])

    EndParams = [minaltitude,maxdistance,maxtime]

    Zenith = zenith
    Azimuth = azimuth

    CreateStations = stations.Stations(Stations, startaltitude, Zenith, Azimuth)
    InputtedStations = CreateStations
    if 'customlocations' in locals() or 'customlocations' in globals():
          CreateStations.AddLocation(customlocations)
    Used_Stations_Temp = CreateStations.GetStations()
    temp = list(Used_Stations_Temp)
    Station_Array = temp

    
    ParticleArray = [Anum,AntiCheck]

    misc.ParamCheck(startaltitude,year,Internal,EndParams)

    ConeInputArray = [RigidityArray,DateArray,MagFieldModel,IntModel,ParticleArray,IOPTinput,WindArray,Magnetopause,coordsystem,gyropercent,EndParams, Station_Array, InputtedStations, KpS, corenum, LiveData, g, h]

    return ConeInputArray


import sys, os
import subprocess
from setuptools import setup, find_packages
from setuptools.command.install import install
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='OTSO',
    version='1.0.15',
    author='Nicholas Larsen',
    author_email='nlarsen1505@gmail.com',
    description='Geomagnetic Cutoff Computation Tool',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/NLarsen15/OTSOpy',
    packages=find_packages(),
    #ext_modules=ext_modules,
    include_package_data=True,
    entry_points={
            'console_scripts': [
                'OTSO.clean=OTSO:clean',
                'OTSO.addstation=OTSO:addstation',
                'OTSO.removestation=OTSO:removestation',
                'OTSO.liststations=OTSO:liststations',
            ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.12, <3.13',
    install_requires=[
        'psutil==7.0.0',     # Common dependency
    ],
    extras_require={
        ':python_version>="3.10"': [
            'numpy>=2.0.0',
            'pandas>=2.2.2',
            'requests==2.32.3',
        ],
    },
    )



import time
from datetime import datetime
import multiprocessing as mp
import os
from . import fortran_calls, readme_generators,cores, misc, flight_inputs
import pandas as pd
import sys
import queue
import numpy as np

def OTSO_flight(latitudes,longitudes,dates,altitudes,cutoff_comp,minaltitude,maxdistance,maxtime,
           serverdata,livedata,vx,vy,vz,by,bz,density,pdyn,Dst,
           G1,G2,G3,W1,W2,W3,W4,W5,W6,kp,Anum,anti,internalmag,externalmag,
           intmodel,startrigidity,endrigidity,rigiditystep,rigidityscan,
           coordsystem,gyropercent,magnetopause,corenum,azimuth,zenith,g,h,asymptotic,asymlevels,unit,MHDfile,MHDcoordsys,spheresize):

    FlightInputArray = flight_inputs.FlightInputs(latitudes,longitudes,dates,altitudes,cutoff_comp,minaltitude,maxdistance,maxtime,
           serverdata,livedata,vx,vy,vz,by,bz,density,pdyn,Dst,
           G1,G2,G3,W1,W2,W3,W4,W5,W6,kp,Anum,anti,internalmag,externalmag,
           intmodel,startrigidity,endrigidity,rigiditystep,rigidityscan,
           coordsystem,gyropercent,magnetopause,corenum,azimuth,zenith,g,h,asymptotic,asymlevels,unit,MHDfile,MHDcoordsys)

    RigidityArray = FlightInputArray[0]
    DateArray = FlightInputArray[1]
    Model = FlightInputArray[2]
    IntModel = FlightInputArray[3]
    ParticleArray = FlightInputArray[4]
    IOPT = FlightInputArray[5]
    WindArray = FlightInputArray[6]
    Magnetopause = FlightInputArray[7]
    CoordinateSystem = FlightInputArray[8]
    MaxStepPercent = FlightInputArray[9]/100
    EndParams = FlightInputArray[10]
    Station_Array = FlightInputArray[11]
    Rcomp = FlightInputArray[12]
    Rscan = FlightInputArray[13]
    KpList = FlightInputArray[14]
    corenum = FlightInputArray[15]
    LiveData = FlightInputArray[16]
    serverdata = FlightInputArray[17]
    g = FlightInputArray[18]
    h = FlightInputArray[19]

    AntiCheck = ParticleArray[1]

    ChildProcesses = []

    UsedCores = cores.Cores(Station_Array, corenum)
    CoreList = UsedCores.getCoreList()
    Positionlists = UsedCores.getPositions()
    WindLists = np.array_split(WindArray, corenum)
    IOPTLists = np.array_split(IOPT, corenum)
    DateArrayLists = np.array_split(DateArray, corenum)


    current_dir = os.path.dirname(os.path.realpath(__file__))
    flight_list = [os.path.join(current_dir, f"Flight{i}.csv") for i in range(1, (corenum) + 1)]
    
    print("OTSO Flight Computation Started")
    start = time.time()
    sys.stdout.write(f"\r{0:.2f}% complete")

    results = []
    resultsfinal = []
    processed = 0
    totalp = 0
    total_stations = len(Station_Array)

    try:
        if not mp.get_start_method(allow_none=True):
             mp.set_start_method('spawn')
    except RuntimeError:
         pass

    ProcessQueue = mp.Manager().Queue()
    for Data, Core, Date, I, Wind, flightFile in zip(Positionlists,CoreList,DateArrayLists, IOPTLists, WindLists, flight_list):
        Child = mp.Process(target=fortran_calls.fortrancallFlight,  args=(Data, RigidityArray, Date, Model, IntModel, 
                                                                              ParticleArray, I, Wind, 
                                                                              Magnetopause, MaxStepPercent, EndParams, 
                                                                              Rcomp, Rscan, asymptotic, asymlevels, unit,
                                                                              ProcessQueue,g,h,CoordinateSystem, flightFile,  MHDfile, MHDcoordsys,
                                                                              spheresize))
        ChildProcesses.append(Child)

    for a in ChildProcesses:
        a.start()

    while processed < total_stations:
        try:
            result_collector = []
            while True:
                try:
                    countint = ProcessQueue.get(timeout=0.001)
                    result_collector.append(countint)
                    processed += 1
                    totalp = totalp + sum(result_collector)
                    result_collector = []
                except queue.Empty:
                    break
    
            percent_complete = (totalp / total_stations) * 100
            sys.stdout.write(f"\r{percent_complete:.2f}% complete")
            sys.stdout.flush()

    
        except queue.Empty:
            pass
      
        time.sleep(0.01)

    for b in ChildProcesses:
        b.join()
        b.close()

    for x in flight_list:
        df = pd.read_csv(x)
        resultsfinal.append(df)
        os.remove(x)

    merged_df = pd.concat(resultsfinal, ignore_index=True)
    merged_df = merged_df.sort_index(axis=0)

    print("\nOTSO Flight Computation Complete")
    stop = time.time()
    Printtime = round((stop-start),3)
    print("Whole Program Took: " + str(Printtime) + " seconds")

    

    readme = readme_generators.READMEFlight(Data, RigidityArray, Model, IntModel, 
                                            AntiCheck, IOPT, WindArray, Magnetopause, Printtime,
                                            MaxStepPercent*100, EndParams, cutoff_comp, Rscan, 
                                            LiveData, asymptotic, asymlevels, unit, serverdata, kp)

    
    datareadme = readme_generators.READMEFlightData(DateArray,WindArray,KpList)
    
    if livedata == 1:
        misc.remove_files()

    return [merged_df,readme,datareadme]


from datetime import date
import os
import shutil
from . import misc
from . import date as d
import pandas as pd

def READMECone(UsedStationstemp, RigidityArray, EventDate, model, IntModel, AtomicNum, AntiCheck, IOPT, WindArray, Magnetopause, CoordinateSystem, Printtime, MaxStepPercent, EndParams, LiveData, serverdata, kp):
    result = []

    pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

    if LiveData == 1:
       OnlineData = "Online Space Weather Data Used"
    elif serverdata == "ON":
       OnlineData = "Server Data Generated Using OMNI Database Used"
    else:
       OnlineData = "User Inputted Data Used"
    
    particle = "anti-particle" if AntiCheck == 1 else "Normal Particle"
    
    IntegrationMethods = ["4th Order Runge-Kutta Method", "Boris Method", "Vay Method", "Higuera-Cary Method"]
    IntegrationMethod = IntegrationMethods[IntModel-1] if 0 <= IntModel <= 4 else "Unknown Integration Method"

    InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
    Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"

    ExternalModels = [
        "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
        "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
    ]
    External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
    if model[1] == 99:
         External = "MHD file"

    PauseModels = [
        "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
        "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
        "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
    ]
    PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"

    today = date.today()
    result.append(f"\n")
    result.append(f"Date of OTSO computation: {today}\n")
    result.append(f"Total computation time: {Printtime} seconds\n\n")
    result.append(f"Output Coordinate System:\n{CoordinateSystem}\n\n")
    result.append(f"Integration Method:\n{IntegrationMethod}\n\n")
    result.append(f"Input Variables:\n\n")
    result.append(f"Data Used: {OnlineData}\n\n")
    result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
    result.append(f"Max Time Step [% of gyrofrequency]: {MaxStepPercent}\n\n")
    result.append(f"Minimum Altitude: {EndParams[0]}km\n")
    result.append(f"Max Distance: {EndParams[1]}Re\n")
    result.append(f"Max Time: {EndParams[2]}s\n\n")
    result.append(f"Start Altitude = {UsedStationstemp[0][3]}km \n")
    result.append(f"Zenith = {UsedStationstemp[0][4]}\n")
    result.append(f"Azimuth = {UsedStationstemp[0][5]}\n\n")
    result.append(f"Kp = {kp}\n")
    result.append(f"IOPT = {IOPT}\n\n")
    result.append(f"Solar Wind Speed [km/s]:\n")
    result.append(f"Vx = {abs(WindArray[0])}\n")
    result.append(f"Vy = {WindArray[1]}\n")
    result.append(f"Vz = {WindArray[2]}\n\n")
    result.append(f"IMF [nT]:\n")
    result.append(f"By = {WindArray[3]}\n")
    result.append(f"Bz = {WindArray[4]}\n\n")
    result.append(f"Density = {WindArray[5]} cm^-3\n")
    result.append(f"Pdyn = {pressure} nPa\n\n")
    result.append(f"Dst = {WindArray[7]} nT\n\n")
    result.append(f"G1 = {WindArray[8]}\n")
    result.append(f"G2 = {WindArray[9]}\n")
    result.append(f"G3 = {WindArray[10]}\n\n")
    result.append(f"W1 = {WindArray[11]}\n")
    result.append(f"W2 = {WindArray[12]}\n")
    result.append(f"W3 = {WindArray[13]}\n")
    result.append(f"W4 = {WindArray[14]}\n")
    result.append(f"W5 = {WindArray[15]}\n")
    result.append(f"W6 = {WindArray[16]}\n\n")
    result.append(f"Atomic Number = {AtomicNum}\n\n")
    result.append(f"Particle Type = {particle}\n\n")
    result.append(f"Magnetic Field Models:\n")
    result.append(f"Internal Model = {Internal}\n")
    result.append(f"External Model = {External}\n\n")
    result.append(f"Magnetopause Model = {PauseModel}\n\n")
    result.append(f"Rigidity:\n")
    result.append(f"Start = {RigidityArray[0]} [GV]\n")
    result.append(f"End = {RigidityArray[1]} [GV]\n")
    result.append(f"Step = {RigidityArray[2]} [GV]\n\n")
    result.append(f"Stations:\n")

    for station in UsedStationstemp:
        result.append(f"{station[0]}, Latitude: {station[1]}, Longitude: {station[2]}\n")

    return "".join(result)

def READMECutoff(UsedStationstemp, RigidityArray, EventDate, model, IntModel, AtomicNum, AntiCheck, IOPT, WindArray, Magnetopause, CoordinateSystem, Printtime, MaxStepPercent, EndParams, Rcomp, Rscan, LiveData, serverdata, kp):
    
    result = []

    pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

    if LiveData == 1:
      OnlineData = "Online Space Weather Data Used"
    elif serverdata == "ON":
       OnlineData = "Server Data Generated Using OMNI Database Used"
    else:
       OnlineData = "User Inputted Data Used"
    
    if Rcomp == "Vertical":
      CutoffComp = "Vertical Cutoff Rigidity"
    elif Rcomp == "Apparent":
      CutoffComp = "Apparent Cutoff Rigidity"
    else:
      CutoffComp = "Custom Cutoff"

    particle = "anti-particle" if AntiCheck == 1 else "Normal Particle"
    RigidityScan = "Rigidity Scan Used" if Rscan != 0 else "No Rigidity Scan"
    
    IntegrationMethods = ["4th Order Runge-Kutta Method", "Boris Method", "Vay Method", "Higuera-Cary Method"]
    IntegrationMethod = IntegrationMethods[IntModel-1] if 0 <= IntModel <= 4 else "Unknown Integration Method"

    InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
    Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"

    ExternalModels = [
        "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
        "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
    ]
    External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
    if model[1] == 99:
         External = "MHD file"

    PauseModels = [
        "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
        "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
        "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
    ]
    PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"

    today = date.today()
    result.append(f"\n")
    result.append(f"Date of OTSO computation: {today}\n")
    result.append(f"Total computation time: {Printtime} seconds\n\n")
    result.append(f"Cutoff Computed: {CutoffComp}\n\n")
    result.append(f"Output Coordinate System:\n{CoordinateSystem}\n\n")
    result.append(f"Rigidity Scan:\n{RigidityScan}\n\n")
    result.append(f"Integration Method:\n{IntegrationMethod}\n\n")
    result.append(f"Input Variables:\n\n")
    result.append(f"Data Used: {OnlineData}\n\n")
    result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
    result.append(f"Max Time Step [% of gyrofrequency]: {MaxStepPercent}\n\n")
    result.append(f"Minimum Altitude: {EndParams[0]}km\n")
    result.append(f"Max Distance: {EndParams[1]}Re\n")
    result.append(f"Max Time: {EndParams[2]}s\n\n")
    result.append(f"Start Altitude = {UsedStationstemp[0][3]}km \n")
    result.append(f"Zenith = {UsedStationstemp[0][4]}\n")
    result.append(f"Azimuth = {UsedStationstemp[0][5]}\n\n")
    result.append(f"Kp = {kp}\n")
    result.append(f"IOPT = {IOPT}\n\n")
    result.append(f"Solar Wind Speed [km/s]:\n")
    result.append(f"Vx = {abs(WindArray[0])}\n")
    result.append(f"Vy = {WindArray[1]}\n")
    result.append(f"Vz = {WindArray[2]}\n\n")
    result.append(f"IMF [nT]:\n")
    result.append(f"By = {WindArray[3]}\n")
    result.append(f"Bz = {WindArray[4]}\n\n")
    result.append(f"Density = {WindArray[5]} cm^-3\n")
    result.append(f"Pdyn = {pressure} nPa\n\n")
    result.append(f"Dst = {WindArray[7]} nT\n\n")
    result.append(f"G1 = {WindArray[8]}\n")
    result.append(f"G2 = {WindArray[9]}\n")
    result.append(f"G3 = {WindArray[10]}\n\n")
    result.append(f"W1 = {WindArray[11]}\n")
    result.append(f"W2 = {WindArray[12]}\n")
    result.append(f"W3 = {WindArray[13]}\n")
    result.append(f"W4 = {WindArray[14]}\n")
    result.append(f"W5 = {WindArray[15]}\n")
    result.append(f"W6 = {WindArray[16]}\n\n")
    result.append(f"Atomic Number = {AtomicNum}\n\n")
    result.append(f"Particle Type = {particle}\n\n")
    result.append(f"Magnetic Field Models:\n")
    result.append(f"Internal Model = {Internal}\n")
    result.append(f"External Model = {External}\n\n")
    result.append(f"Magnetopause Model = {PauseModel}\n\n")
    result.append(f"Rigidity:\n")
    result.append(f"Start = {RigidityArray[0]} [GV]\n")
    result.append(f"End = {RigidityArray[1]} [GV]\n")
    result.append(f"Step = {RigidityArray[2]} [GV]\n\n")
    result.append(f"Stations:\n")

    for station in UsedStationstemp:
        result.append(f"{station[0]}, Latitude: {station[1]}, Longitude: {station[2]}\n")

    return "".join(result)

def READMEFlight(Data, Rigidity, model, IntModel, AntiCheck, IOPT, WindArray, Magnetopause, Printtime, MaxStepPercent, EndParams, Rcomp, Rscan, LiveData, asymptotic, asymlevels, unit, serverdata, kp):
      result = []

      if LiveData == 1:
         OnlineData = "Online Space Weather Data Used"
      elif serverdata == "ON":
         OnlineData = "Server Data Generated Using OMNI Database Used"
      else:
         OnlineData = "User Inputted Data Used"
      
      if Rcomp == "Vertical":
         CutoffComp = "Vertical Cutoff Rigidity"
      elif Rcomp == "Apparent":
         CutoffComp = "Apparent Cutoff Rigidity"
      else:
         CutoffComp = "Custom Cutoff"
      
      particle = "anti-particle" if AntiCheck == 1 else "Normal Particle"
      RigidityScan = "Rigidity Scan Used" if Rscan != 0 else "No Rigidity Scan"
      
      IntegrationMethods = ["4th Order Runge-Kutta Method", "Boris Method", "Vay Method", "Higuera-Cary Method"]
      IntegrationMethod = IntegrationMethods[IntModel-1] if 0 <= IntModel <= 4 else "Unknown Integration Method"
  
      InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
      Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"
  
      ExternalModels = [
          "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
          "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
      ]
      External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
      if model[1] == 99:
         External = "MHD file"
  
      PauseModels = [
          "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
          "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
          "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
      ]
      PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"
  
      today = date.today()
      result.append(f"\n")
      result.append(f"Date of OTSO computation: {today}\n")
      result.append(f"Total computation time: {Printtime} seconds\n\n")
      result.append(f"Cutoff Computed: {CutoffComp}\n\n")
      result.append(f"Integration Method:\n{IntegrationMethod}\n\n")
      result.append(f"Rigidity Scan:\n{RigidityScan}\n\n")
      result.append(f"Input Variables:\n\n")
      result.append(f"Data Used: {OnlineData}\n\n")
      result.append(f"Max Time Step [% of gyrofrequency]: {MaxStepPercent}\n\n")
      result.append(f"Minimum Altitude: {EndParams[0]}km\n")
      result.append(f"Max Distance: {EndParams[1]}Re\n")
      result.append(f"Max Time: {EndParams[2]}s\n\n")
      result.append(f"Zenith = {Data[0][4]}\n")
      result.append(f"Azimuth = {Data[0][5]}\n\n")
      result.append(f"Particle Type = {particle}\n\n")
      result.append(f"Magnetic Field Models:\n")
      result.append(f"Internal Model = {Internal}\n")
      result.append(f"External Model = {External}\n\n")
      result.append(f"Magnetopause Model = {PauseModel}\n\n")
      result.append(f"Rigidity:\n")
      result.append(f"Start = {Rigidity[0]} [GV]\n")
      result.append(f"End = {Rigidity[1]} [GV]\n")
      result.append(f"Step = {Rigidity[2]} [GV]\n\n")
      if asymptotic == "YES":
         result.append("Asymptotic Directions: " + str(asymptotic) + " \n")
         result.append("Asymptotic Levels: " + str(asymlevels) + " \n")
         result.append("Asymptotic Levels Unit: " + str(unit) + " \n")
    
      return "".join(result)

def READMEFlightData(DateArray,WindArray,kparray):

   headers = ["Date","kp","Vx [km/s]", "Vy [km/s]", "Vz [km/s]", "By [nT]", "Bz [nT]", "Density [cm^-3]", "Pdyn [nPa]", "Dst [nT]", "G1", "G2",
              "G3", "W1", "W2", "W3", "W4", "W5", "W6"]
   
   rows = []

   for date,Wind,kp in zip(DateArray,WindArray,kparray):
      datetimeobj = d.convert_to_datetime(date)
      pressure = misc.Pdyn_comp(Wind[5],abs(Wind[0]))
      Vx = abs(Wind[0])
      Vy = Wind[1]
      Vz = Wind[2]
      By = Wind[3]
      Bz = Wind[4]
      Density = Wind[5]
      Pdyn = pressure
      Dst = Wind[7]
      G1 = Wind[8]
      G2 = Wind[9]
      G3 = Wind[10]
      W1 = Wind[11]
      W2 = Wind[12]
      W3 = Wind[13]
      W4 = Wind[14]
      W5 = Wind[15]
      W6 = Wind[16]
      data = {
      "Date": datetimeobj,
      "kp": kp,
      "Vx [km/s]": Vx,
      "Vy [km/s]": Vy,
      "Vz [km/s]": Vz,
      "By [nT]": By,
      "Bz [nT]": Bz,
      "Density [cm^-3]": Density,
      "Pdyn [nPa]": Pdyn,
      "Dst [nT]": Dst,
      "G1": G1,
      "G2": G2,
      "G3": G3,
      "W1": W1,
      "W2": W2,
      "W3": W3,
      "W4": W4,
      "W5": W5,
      "W6": W6,}
      
      rows.append(data)

   df = pd.DataFrame(rows, columns=headers)

   return df

def READMETrajectory(UsedStationstemp, Rigidity, EventDate, model, IntModel, AtomicNum, AntiCheck, IOPT, WindArray, Magnetopause, CoordinateSystem, Printtime, MaxStepPercent, EndParams, LiveData, kp, serverdata):
    result = []

    pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

    if LiveData == 1:
      OnlineData = "Online Space Weather Data Used"
    elif serverdata == "ON":
       OnlineData = "Server Data Generated Using OMNI Database Used"
    else:
       OnlineData = "User Inputted Data Used"

    particle = "anti-particle" if AntiCheck == 1 else "Normal Particle"
    
    IntegrationMethods = ["4th Order Runge-Kutta Method", "Boris Method", "Vay Method", "Higuera-Cary Method"]
    IntegrationMethod = IntegrationMethods[IntModel-1] if 0 <= IntModel <= 4 else "Unknown Integration Method"

    InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
    Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"

    ExternalModels = [
        "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
        "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
    ]
    External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
    if model[1] == 99:
         External = "MHD file"

    PauseModels = [
        "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
        "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
        "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
    ]
    PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"

    today = date.today()
    result.append(f"\n")
    result.append(f"Date of OTSO computation: {today}\n")
    result.append(f"Total computation time: {Printtime} seconds\n\n")
    result.append(f"Output Coordinate System:\n{CoordinateSystem}\n\n")
    result.append(f"Integration Method:\n{IntegrationMethod}\n\n")
    result.append(f"Input Variables:\n\n")
    result.append(f"Data Used: {OnlineData}\n\n")
    result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
    result.append(f"Max Time Step [% of gyrofrequency]: {MaxStepPercent}\n\n")
    result.append(f"Minimum Altitude: {EndParams[0]}km\n")
    result.append(f"Max Distance: {EndParams[1]}Re\n")
    result.append(f"Max Time: {EndParams[2]}s\n\n")
    result.append(f"Start Altitude = {UsedStationstemp[0][3]}km \n")
    result.append(f"Zenith = {UsedStationstemp[0][4]}\n")
    result.append(f"Azimuth = {UsedStationstemp[0][5]}\n\n")
    result.append(f"Kp = {kp}\n")
    result.append(f"IOPT = {IOPT}\n\n")
    result.append(f"Solar Wind Speed [km/s]:\n")
    result.append(f"Vx = {abs(WindArray[0])}\n")
    result.append(f"Vy = {WindArray[1]}\n")
    result.append(f"Vz = {WindArray[2]}\n\n")
    result.append(f"IMF [nT]:\n")
    result.append(f"By = {WindArray[3]}\n")
    result.append(f"Bz = {WindArray[4]}\n\n")
    result.append(f"Density = {WindArray[5]} cm^-3\n")
    result.append(f"Pdyn = {pressure} nPa\n\n")
    result.append(f"Dst = {WindArray[7]} nT\n\n")
    result.append(f"G1 = {WindArray[8]}\n")
    result.append(f"G2 = {WindArray[9]}\n")
    result.append(f"G3 = {WindArray[10]}\n\n")
    result.append(f"W1 = {WindArray[11]}\n")
    result.append(f"W2 = {WindArray[12]}\n")
    result.append(f"W3 = {WindArray[13]}\n")
    result.append(f"W4 = {WindArray[14]}\n")
    result.append(f"W5 = {WindArray[15]}\n")
    result.append(f"W6 = {WindArray[16]}\n\n")
    result.append(f"Atomic Number = {AtomicNum}\n\n")
    result.append(f"Particle Type = {particle}\n\n")
    result.append(f"Magnetic Field Models:\n")
    result.append(f"Internal Model = {Internal}\n")
    result.append(f"External Model = {External}\n\n")
    result.append(f"Magnetopause Model = {PauseModel}\n\n")
    result.append(f"Rigidity = {Rigidity}\n\n")
    result.append(f"Stations:\n")

    for station in UsedStationstemp:
        result.append(f"{station[0]}, Latitude: {station[1]}, Longitude: {station[2]}\n")

    return "".join(result)

def READMEPlanet(Data, Rigidity, EventDate, model, IntModel, AntiCheck, IOPT, WindArray, Magnetopause, Printtime,maxlat,maxlong,minlat,minlong,LatStep, LongStep, MaxStepPercent, EndParams, Rcomp, Rscan, LiveData, asymptotic, asymlevels, unit, serverdata, kp, custom_coords_provided=False):
      result = []

      pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

      if LiveData == 1:
         OnlineData = "Online Space Weather Data Used"
      elif serverdata == "ON":
         OnlineData = "Server Data Generated Using OMNI Database Used"
      else:
         OnlineData = "User Inputted Data Used"
      
      if Rcomp == "Vertical":
         CutoffComp = "Vertical Cutoff Rigidity"
      elif Rcomp == "Apparent":
         CutoffComp = "Apparent Cutoff Rigidity"
      else:
         CutoffComp = "Custom Cutoff"
      
      particle = "anti-particle" if AntiCheck == 1 else "Normal Particle"
      RigidityScan = "Rigidity Scan Used" if Rscan != 0 else "No Rigidity Scan"
      
      IntegrationMethods = ["4th Order Runge-Kutta Method", "Boris Method", "Vay Method", "Higuera-Cary Method"]
      IntegrationMethod = IntegrationMethods[IntModel-1] if 0 <= IntModel <= 4 else "Unknown Integration Method"
  
      InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
      Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"
  
      ExternalModels = [
          "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
          "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
      ]
      External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
      if model[1] == 99:
         External = "MHD file"
      
  
      PauseModels = [
          "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
          "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
          "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
      ]
      PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"
  
      today = date.today()
      result.append(f"\n")
      result.append(f"Date of OTSO computation: {today}\n")
      result.append(f"Total computation time: {Printtime} seconds\n\n")
      result.append(f"Cutoff Computed: {CutoffComp}\n\n")
      result.append(f"Integration Method:\n{IntegrationMethod}\n\n")
      result.append(f"Rigidity Scan:\n{RigidityScan}\n\n")
      result.append(f"Input Variables:\n\n")
      result.append(f"Data Used: {OnlineData}\n\n")
      result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
      result.append(f"Max Time Step [% of gyrofrequency]: {MaxStepPercent}\n\n")
      result.append(f"Minimum Altitude: {EndParams[0]}km\n")
      result.append(f"Max Distance: {EndParams[1]}Re\n")
      result.append(f"Max Time: {EndParams[2]}s\n\n")
      
      if Data and len(Data[0]) > 5:
          result.append(f"Start Altitude = {Data[0][3]}km \n")
          result.append(f"Zenith = {Data[0][4]}\n")
          result.append(f"Azimuth = {Data[0][5]}\n\n")
      else:
          result.append(f"Start Altitude = Not Available\n")
          result.append(f"Zenith = Not Available\n")
          result.append(f"Azimuth = Not Available\n\n")

      result.append(f"Kp = {kp}\n")
      result.append(f"IOPT = {IOPT}\n\n")
      result.append(f"Solar Wind Speed [km/s]:\n")
      result.append(f"Vx = {abs(WindArray[0])}\n")
      result.append(f"Vy = {WindArray[1]}\n")
      result.append(f"Vz = {WindArray[2]}\n\n")
      result.append(f"IMF [nT]:\n")
      result.append(f"By = {WindArray[3]}\n")
      result.append(f"Bz = {WindArray[4]}\n\n")
      result.append(f"Density = {WindArray[5]} cm^-3\n")
      result.append(f"Pdyn = {pressure} nPa\n\n")
      result.append(f"Dst = {WindArray[7]} nT\n\n")
      result.append(f"G1 = {WindArray[8]}\n")
      result.append(f"G2 = {WindArray[9]}\n")
      result.append(f"G3 = {WindArray[10]}\n\n")
      result.append(f"W1 = {WindArray[11]}\n")
      result.append(f"W2 = {WindArray[12]}\n")
      result.append(f"W3 = {WindArray[13]}\n")
      result.append(f"W4 = {WindArray[14]}\n")
      result.append(f"W5 = {WindArray[15]}\n")
      result.append(f"W6 = {WindArray[16]}\n\n")
      result.append(f"Particle Type = {particle}\n\n")
      result.append(f"Magnetic Field Models:\n")
      result.append(f"Internal Model = {Internal}\n")
      result.append(f"External Model = {External}\n\n")
      result.append(f"Magnetopause Model = {PauseModel}\n\n")
      result.append(f"Rigidity:\n")
      result.append(f"Start = {Rigidity[0]} [GV]\n")
      result.append(f"End = {Rigidity[1]} [GV]\n")
      result.append(f"Step = {Rigidity[2]} [GV]\n\n")

      if custom_coords_provided:
          result.append("Coordinate Input: Custom list of (Latitude, Longitude) pairs provided.\n\n")
      else:
          result.append("Max and Min Latitude and Longitude:"+ "\n")
          result.append("Latitude: Max = " + str(maxlat) + " Min = " + str(minlat) + "\n")
          result.append("Longitude: Max = " + str(maxlong) + " Min = " + str(minlong) + "\n\n")
          result.append("Latitude and Longitude Steps:"+ "\n")
          lat_step_str = str(abs(LatStep)) if LatStep is not None else "N/A"
          long_step_str = str(abs(LongStep)) if LongStep is not None else "N/A"
          result.append("Latitude = " + lat_step_str + " degree steps" + "\n")
          result.append("Longitude = " + long_step_str + " degree steps" + "\n\n")
      
      if asymptotic == "YES":
         result.append("Asymptotic Directions: " + str(asymptotic) + " \n")
         result.append("Asymptotic Levels: " + str(asymlevels) + " \n")
         result.append("Asymptotic Levels Unit: " + str(unit) + " \n")
    
      return "".join(result)

def READMEMagfield(EventDate, model, IOPT, WindArray, CoordinateSystem, Printtime, LiveData, serverdata,kp):
    result = []

    pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

    if LiveData == 1:
      OnlineData = "Online Space Weather Data Used"
    elif serverdata == "ON":
       OnlineData = "Server Data Generated Using OMNI Database Used"
    else:
       OnlineData = "User Inputted Data Used"

    InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
    Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"

    ExternalModels = [
        "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
        "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
    ]
    External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
    if model[1] == 99:
         External = "MHD file"

    today = date.today()
    result.append(f"\n")
    result.append(f"Date of OTSO computation: {today}\n")
    result.append(f"Total computation time: {Printtime} seconds\n\n")
    result.append(f"Input Coordinate System:\n{CoordinateSystem}\n\n")
    result.append(f"Input Variables:\n\n")
    result.append(f"Data Used: {OnlineData}\n\n")
    result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
    result.append(f"Kp = {kp}\n")
    result.append(f"IOPT = {IOPT}\n\n")
    result.append(f"Solar Wind Speed [km/s]:\n")
    result.append(f"Vx = {abs(WindArray[0])}\n")
    result.append(f"Vy = {WindArray[1]}\n")
    result.append(f"Vz = {WindArray[2]}\n\n")
    result.append(f"IMF [nT]:\n")
    result.append(f"By = {WindArray[3]}\n")
    result.append(f"Bz = {WindArray[4]}\n\n")
    result.append(f"Density = {WindArray[5]} cm^-3\n")
    result.append(f"Pdyn = {pressure} nPa\n\n")
    result.append(f"Dst = {WindArray[7]} nT\n\n")
    result.append(f"G1 = {WindArray[8]}\n")
    result.append(f"G2 = {WindArray[9]}\n")
    result.append(f"G3 = {WindArray[10]}\n\n")
    result.append(f"W1 = {WindArray[11]}\n")
    result.append(f"W2 = {WindArray[12]}\n")
    result.append(f"W3 = {WindArray[13]}\n")
    result.append(f"W4 = {WindArray[14]}\n")
    result.append(f"W5 = {WindArray[15]}\n")
    result.append(f"W6 = {WindArray[16]}\n\n")
    result.append(f"Magnetic Field Models:\n")
    result.append(f"Internal Model = {Internal}\n")
    result.append(f"External Model = {External}\n\n")

    return "".join(result)

def READMECoordtrans(CoordIN, CoordOUT, Printtime):
    result = []

    today = date.today()
    result.append(f"\n")
    result.append(f"Date of OTSO computation: {today}\n")
    result.append(f"Total computation time: {Printtime} seconds\n\n")
    result.append(f"Input Coordinate System:\n{CoordIN}\n\n")
    result.append(f"Output Coordinate System:\n{CoordOUT}\n\n")
    

    return "".join(result)

def READMETrace(Data, EventDate, model, IntModel, AntiCheck, IOPT, WindArray, Magnetopause, Printtime,maxlat,maxlong,minlat,minlong,LatStep, LongStep, MaxStepPercent, EndParams, LiveData, serverdata, kp):
      result = []

      pressure = misc.Pdyn_comp(WindArray[5],abs(WindArray[0]))

      if LiveData == 1:
         OnlineData = "Online Space Weather Data Used"
      elif serverdata == "ON":
         OnlineData = "Server Data Generated Using OMNI Database Used"
      else:
         OnlineData = "User Inputted Data Used"
  
      InternalModels = ["None", "IGRF", "Dipole", "Custom Gaussian Coefficients", "Custom Gaussian Coefficients"]
      Internal = InternalModels[model[0]] if 0 <= model[0] <= 4 else "Unknown Internal Model"
  
      ExternalModels = [
          "No External Field", "Tsyganenko 87 Short", "Tsyganenko 87 Long", "Tsyganenko 89",
          "Tsyganenko 96", "Tsyganenko 01", "Tsyganenko 01 Storm", "Tsyganenko 04", "Tsyganenko 89 Boberg Extension"
      ]
      External = ExternalModels[model[1]] if 0 <= model[1] <= 8 else "Unknown External Model"
      if model[1] == 99:
         External = "MHD file"
  
      PauseModels = [
          "25Re Sphere", "Aberrated Formisano Model", "Sibeck Model", "Kobel Model",
          "Tsyganenko 96 Magnetopause Model", "Tsyganenko 01 Magnetopause Model",
          "Tsyganenko 01 Storm Magnetopause Model", "Tsyganenko 04 Magnetopause Model"
      ]
      PauseModel = PauseModels[Magnetopause] if 0 <= Magnetopause <= 7 else "Unknown Magnetopause Model"
  
      today = date.today()
      result.append(f"\n")
      result.append(f"Date of OTSO computation: {today}\n")
      result.append(f"Total computation time: {Printtime} seconds\n\n")
      result.append(f"Input Variables:\n\n")
      result.append(f"Data Used: {OnlineData}\n\n")
      result.append(f"Simulation Date: {EventDate.strftime('%d/%m/%Y, %H:%M:%S')}\n\n")
      result.append(f"Kp = {kp}\n")
      result.append(f"IOPT = {IOPT}\n\n")
      result.append(f"Solar Wind Speed [km/s]:\n")
      result.append(f"Vx = {abs(WindArray[0])}\n")
      result.append(f"Vy = {WindArray[1]}\n")
      result.append(f"Vz = {WindArray[2]}\n\n")
      result.append(f"IMF [nT]:\n")
      result.append(f"By = {WindArray[3]}\n")
      result.append(f"Bz = {WindArray[4]}\n\n")
      result.append(f"Density = {WindArray[5]} cm^-3\n")
      result.append(f"Pdyn = {pressure} nPa\n\n")
      result.append(f"Dst = {WindArray[7]} nT\n\n")
      result.append(f"G1 = {WindArray[8]}\n")
      result.append(f"G2 = {WindArray[9]}\n")
      result.append(f"G3 = {WindArray[10]}\n\n")
      result.append(f"W1 = {WindArray[11]}\n")
      result.append(f"W2 = {WindArray[12]}\n")
      result.append(f"W3 = {WindArray[13]}\n")
      result.append(f"W4 = {WindArray[14]}\n")
      result.append(f"W5 = {WindArray[15]}\n")
      result.append(f"W6 = {WindArray[16]}\n\n")
      result.append(f"Magnetic Field Models:\n")
      result.append(f"Internal Model = {Internal}\n")
      result.append(f"External Model = {External}\n\n")
      result.append(f"Magnetopause Model = {PauseModel}\n\n")
      result.append("Max and Min Latitude and Longitude:"+ "\n")
      result.append("Latitude: Max = " + str(maxlat) + " Min = " + str(minlat) + "\n")
      result.append("Longitude: Max = " + str(maxlong) + " Min = " + str(minlong) + "\n\n")
      result.append("Latitude and Longitude Steps:"+ "\n")
      result.append("Latitude = " + str(abs(LatStep)) + " degree steps" + "\n")
      result.append("Longitude = " + str(abs(LongStep)) + " degree steps" + "\n\n")
    
      return "".join(result)


import math


def TSY01_Constants(By,Bz,V,N):
    G1 = 0
    G2 = 0
    G3 = 0

    By = By
    Bz = Bz
    V = V
    W = 1

    B = (By*By + Bz*Bz)**(0.5)
    h = (((B/40)**(2))/(1 + B/40))

    if(By == 0 and Bz == 0):
        phi = 0
    else:
        phi = math.atan2(By,Bz)
        if(phi <= 0):
            phi = phi + 2*math.pi

    if(Bz < 0):
        Bs = abs(Bz)
    elif Bz >= 0:
        Bs = 0
    
    G1 = G1 + (W)*V*h*(math.sin(phi/2)**3)
    G2 = G2 + (0.005)*(W)*(V)*Bs
    G3 = G3 + (N*V*Bs)/2000

    return G1, G2, G3
    